"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.wrappers_pb2
import qwak.administration.v0.users.user_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class UserProfile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    DISPLAY_NAME_FIELD_NUMBER: builtins.int
    JOB_FUNCTION_FIELD_NUMBER: builtins.int
    JOB_LEVEL_FIELD_NUMBER: builtins.int
    ACCOUNT_ROLE_FIELD_NUMBER: builtins.int
    ENVIRONMENT_IDS_FIELD_NUMBER: builtins.int
    TIME_ZONE_FIELD_NUMBER: builtins.int
    @property
    def id(self) -> google.protobuf.wrappers_pb2.StringValue:
        """User ID"""
    @property
    def display_name(self) -> google.protobuf.wrappers_pb2.StringValue:
        """Display user name"""
    job_function: qwak.administration.v0.users.user_pb2.QwakUserJobFunction.ValueType
    """Job function"""
    job_level: qwak.administration.v0.users.user_pb2.QwakUserJobLevel.ValueType
    """Job level"""
    account_role: qwak.administration.v0.users.user_pb2.QwakAccountRole.ValueType
    """User account role"""
    @property
    def environment_ids(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """User bound environments"""
    time_zone: builtins.str
    """Time zone"""
    def __init__(
        self,
        *,
        id: google.protobuf.wrappers_pb2.StringValue | None = ...,
        display_name: google.protobuf.wrappers_pb2.StringValue | None = ...,
        job_function: qwak.administration.v0.users.user_pb2.QwakUserJobFunction.ValueType = ...,
        job_level: qwak.administration.v0.users.user_pb2.QwakUserJobLevel.ValueType = ...,
        account_role: qwak.administration.v0.users.user_pb2.QwakAccountRole.ValueType = ...,
        environment_ids: collections.abc.Iterable[builtins.str] | None = ...,
        time_zone: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["display_name", b"display_name", "id", b"id"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["account_role", b"account_role", "display_name", b"display_name", "environment_ids", b"environment_ids", "id", b"id", "job_function", b"job_function", "job_level", b"job_level", "time_zone", b"time_zone"]) -> None: ...

global___UserProfile = UserProfile



from dataclasses import dataclass
from typing import Optional

from _qwak_proto.qwak.feature_store.sources.batch_pb2 import (
    BatchSource as ProtoBatchSource,
    MongoSource as ProtoMongoSource,
)
from _qwak_proto.qwak.feature_store.sources.data_source_pb2 import (
    DataSourceSpec as ProtoDataSourceSpec,
)
from qwak.feature_store.data_sources.batch._batch import BaseBatchSource


@dataclass
class MongoDbSource(BaseBatchSource):
    hosts: str
    username_secret_name: str
    password_secret_name: str
    database: str
    collection: str
    connection_params: str
    protocol: str = "mongodb"
    repository: Optional[str] = None

    @classmethod
    def _from_proto(cls, proto):
        mongo = proto.mongoSource
        return cls(
            name=proto.name,
            date_created_column=proto.date_created_column,
            description=proto.description,
            hosts=mongo.hosts,
            username_secret_name=mongo.username_secret_name,
            password_secret_name=mongo.password_secret_name,
            database=mongo.database,
            collection=mongo.collection,
            connection_params=mongo.connection_params,
            protocol=mongo.protocol,
        )

    def _to_proto(self, artifact_url: Optional[str] = None):
        return ProtoDataSourceSpec(
            data_source_repository_name=self.repository,
            batch_source=ProtoBatchSource(
                name=self.name,
                description=self.description,
                date_created_column=self.date_created_column,
                mongoSource=ProtoMongoSource(
                    hosts=self.hosts,
                    username_secret_name=self.username_secret_name,
                    password_secret_name=self.password_secret_name,
                    database=self.database,
                    collection=self.collection,
                    connection_params=self.connection_params,
                    protocol=self.protocol,
                ),
            ),
        )



import pandas as pd
from dateutil.parser import ParserError
from qwak.exceptions import QwakException
from qwak.feature_store.offline.feature_set_features import FeatureSetFeatures


def validate_point_in_time_column_in_population(
    population: "pd.DataFrame", point_in_time_column_name: str
):
    if point_in_time_column_name not in population:
        raise QwakException(
            f"The point in time column: `{point_in_time_column_name}` must be part of the population dataframe, current columns: {population.columns}"
        )

    from pandas.api.types import is_datetime64_any_dtype

    if not is_datetime64_any_dtype(population[point_in_time_column_name]):
        try:
            population[point_in_time_column_name] = pd.to_datetime(
                population[point_in_time_column_name]
            )
        except ParserError as e:
            raise QwakException(
                f"It was not possible to cast provided point in time column to datetime"
                f"\nError message: {e}"
            )


def validate_features(featureset_features: FeatureSetFeatures):
    if not featureset_features.feature_set_name:
        raise QwakException("features.feature_set_name must be set")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/model_group/model_group_repository_details.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5qwak/model_group/model_group_repository_details.proto\x12\x1cqwak.model_groups.management\"r\n\x11RepositoryDetails\x12\x45\n\x0frepository_type\x18\x01 \x01(\x0b\x32,.qwak.model_groups.management.RepositoryType\x12\x16\n\x0erepository_key\x18\x02 \x01(\t\"\x80\x03\n\x17RemoteRepositoryDetails\x12K\n\x11\x64ocker_repository\x18\x01 \x01(\x0b\x32..qwak.model_groups.management.DockerRepositoryH\x00\x12V\n\x17hugging_face_repository\x18\x02 \x01(\x0b\x32\x33.qwak.model_groups.management.HuggingFaceRepositoryH\x00\x12\x45\n\x0enpm_repository\x18\x04 \x01(\x0b\x32+.qwak.model_groups.management.NpmRepositoryH\x00\x12G\n\x0fpypi_repository\x18\x05 \x01(\x0b\x32,.qwak.model_groups.management.PypiRepositoryH\x00\x12\x1d\n\x15repository_remote_url\x18\x03 \x01(\tB\x11\n\x0frepository_type\"\x12\n\x10\x44ockerRepository\"\x13\n\x11\x44\x61tasetRepository\"\x14\n\x12\x41rtifactRepository\"\x17\n\x15HuggingFaceRepository\"\x0f\n\rNpmRepository\"\x10\n\x0ePypiRepository\"\xed\x03\n\x0eRepositoryType\x12K\n\x11\x64ocker_repository\x18\x01 \x01(\x0b\x32..qwak.model_groups.management.DockerRepositoryH\x00\x12M\n\x12\x64\x61taset_repository\x18\x02 \x01(\x0b\x32/.qwak.model_groups.management.DatasetRepositoryH\x00\x12O\n\x13\x61rtifact_repository\x18\x03 \x01(\x0b\x32\x30.qwak.model_groups.management.ArtifactRepositoryH\x00\x12V\n\x17hugging_face_repository\x18\x04 \x01(\x0b\x32\x33.qwak.model_groups.management.HuggingFaceRepositoryH\x00\x12\x45\n\x0enpm_repository\x18\x05 \x01(\x0b\x32+.qwak.model_groups.management.NpmRepositoryH\x00\x12G\n\x0fpypi_repository\x18\x06 \x01(\x0b\x32,.qwak.model_groups.management.PypiRepositoryH\x00\x42\x06\n\x04typeBN\n&com.qwak.ai.management.model.group.apiB\"ModelGroupRepositoriesDetailsProtoP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.model_group.model_group_repository_details_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n&com.qwak.ai.management.model.group.apiB\"ModelGroupRepositoriesDetailsProtoP\001'
  _globals['_REPOSITORYDETAILS']._serialized_start=87
  _globals['_REPOSITORYDETAILS']._serialized_end=201
  _globals['_REMOTEREPOSITORYDETAILS']._serialized_start=204
  _globals['_REMOTEREPOSITORYDETAILS']._serialized_end=588
  _globals['_DOCKERREPOSITORY']._serialized_start=590
  _globals['_DOCKERREPOSITORY']._serialized_end=608
  _globals['_DATASETREPOSITORY']._serialized_start=610
  _globals['_DATASETREPOSITORY']._serialized_end=629
  _globals['_ARTIFACTREPOSITORY']._serialized_start=631
  _globals['_ARTIFACTREPOSITORY']._serialized_end=651
  _globals['_HUGGINGFACEREPOSITORY']._serialized_start=653
  _globals['_HUGGINGFACEREPOSITORY']._serialized_end=676
  _globals['_NPMREPOSITORY']._serialized_start=678
  _globals['_NPMREPOSITORY']._serialized_end=693
  _globals['_PYPIREPOSITORY']._serialized_start=695
  _globals['_PYPIREPOSITORY']._serialized_end=711
  _globals['_REPOSITORYTYPE']._serialized_start=714
  _globals['_REPOSITORYTYPE']._serialized_end=1207
# @@protoc_insertion_point(module_scope)



import re
from abc import ABC, abstractmethod
from dataclasses import dataclass
from enum import Enum

from _qwak_proto.qwak.automation.v1.action_pb2 import Action as ActionProto, MemoryUnit
from _qwak_proto.qwak.automation.v1.common_pb2 import MetricThresholdDirection
from _qwak_proto.qwak.user_application.common.v0.resources_pb2 import (
    MemoryUnit as CommonMemoryUnit,
)


@dataclass
class Action(ABC):
    @abstractmethod
    def to_proto(self):
        # abstract method
        pass

    @staticmethod
    @abstractmethod
    def from_proto(message: ActionProto):
        # abstract method
        pass


def map_memory_units(memory: str):
    memory_unit = re.sub(r"\d+", "", memory)
    if memory_unit == "Gi":
        return MemoryUnit.GIB
    elif memory_unit == "Mib":
        return MemoryUnit.MIB
    else:
        return MemoryUnit.UNKNOWN


def map_common_memory_units(memory: str):
    memory_unit = re.sub(r"\d+", "", memory)
    if memory_unit == "Gi":
        return CommonMemoryUnit.GIB
    elif memory_unit == "Mib":
        return CommonMemoryUnit.MIB
    else:
        return CommonMemoryUnit.INVALID_MEMORY_UNIT


def map_memory_units_proto(memory_unit: MemoryUnit):
    if memory_unit == MemoryUnit.MIB:
        return "Mib"
    elif memory_unit == MemoryUnit.GIB:
        return "Gi"
    else:
        return ""


def map_common_memory_units_proto(memory_unit: CommonMemoryUnit):
    if memory_unit == CommonMemoryUnit.MIB:
        return "Mib"
    elif memory_unit == CommonMemoryUnit.GIB:
        return "Gi"
    else:
        return ""


def get_memory_amount(memory: str):
    return int(
        re.sub(
            r"\D",
            "",
            memory,
        )
    )


class ThresholdDirection(Enum):
    ABOVE = 1
    BELOW = 2
    EQUALS = 3
    BELOW_OR_EQUALS = 4
    ABOVE_OR_EQUALS = 5


threshold_to_proto_mapping = {
    ThresholdDirection.ABOVE: MetricThresholdDirection.ABOVE,
    ThresholdDirection.BELOW: MetricThresholdDirection.BELOW,
    ThresholdDirection.EQUALS: MetricThresholdDirection.EQUALS,
    ThresholdDirection.BELOW_OR_EQUALS: MetricThresholdDirection.BELOW_OR_EQUALS,
    ThresholdDirection.ABOVE_OR_EQUALS: MetricThresholdDirection.ABOVE_OR_EQUALS,
}

proto_threshold_to_threshold = {v: k for k, v in threshold_to_proto_mapping.items()}


def map_threshold_direction_to_proto(
    direction: ThresholdDirection,
) -> MetricThresholdDirection:
    return threshold_to_proto_mapping.get(
        direction, MetricThresholdDirection.INVALID_METRIC_DIRECTION
    )


def map_proto_threshold_to_direction(
    direction: MetricThresholdDirection,
) -> ThresholdDirection:
    return proto_threshold_to_threshold.get(direction)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.administration.runtime_configuration.v0.creds.secret_pb2
import sys
import typing

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class DatabricksAuth(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DB_ACCESS_TOKEN_FIELD_NUMBER: builtins.int
    DB_PRINCIPAL_CREDENTIALS_FIELD_NUMBER: builtins.int
    @property
    def db_access_token(self) -> global___DatabricksAccessToken: ...
    @property
    def db_principal_credentials(self) -> global___DatabricksServicePrincipalCredentials: ...
    def __init__(
        self,
        *,
        db_access_token: global___DatabricksAccessToken | None = ...,
        db_principal_credentials: global___DatabricksServicePrincipalCredentials | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["auth_type", b"auth_type", "db_access_token", b"db_access_token", "db_principal_credentials", b"db_principal_credentials"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["auth_type", b"auth_type", "db_access_token", b"db_access_token", "db_principal_credentials", b"db_principal_credentials"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["auth_type", b"auth_type"]) -> typing_extensions.Literal["db_access_token", "db_principal_credentials"] | None: ...

global___DatabricksAuth = DatabricksAuth

class DatabricksServicePrincipalCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CLIENT_ID_FIELD_NUMBER: builtins.int
    CLIENT_SECRET_FIELD_NUMBER: builtins.int
    DB_WORKSPACE_DESCRIPTOR_FIELD_NUMBER: builtins.int
    @property
    def client_id(self) -> qwak.administration.runtime_configuration.v0.creds.secret_pb2.SecretKeyRef: ...
    @property
    def client_secret(self) -> qwak.administration.runtime_configuration.v0.creds.secret_pb2.SecretKeyRef: ...
    @property
    def db_workspace_descriptor(self) -> global___DatabricksWorkspaceDescriptor: ...
    def __init__(
        self,
        *,
        client_id: qwak.administration.runtime_configuration.v0.creds.secret_pb2.SecretKeyRef | None = ...,
        client_secret: qwak.administration.runtime_configuration.v0.creds.secret_pb2.SecretKeyRef | None = ...,
        db_workspace_descriptor: global___DatabricksWorkspaceDescriptor | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["client_id", b"client_id", "client_id_location", b"client_id_location", "client_secret", b"client_secret", "client_secret_location", b"client_secret_location", "db_workspace_descriptor", b"db_workspace_descriptor"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["client_id", b"client_id", "client_id_location", b"client_id_location", "client_secret", b"client_secret", "client_secret_location", b"client_secret_location", "db_workspace_descriptor", b"db_workspace_descriptor"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["client_id_location", b"client_id_location"]) -> typing_extensions.Literal["client_id"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["client_secret_location", b"client_secret_location"]) -> typing_extensions.Literal["client_secret"] | None: ...

global___DatabricksServicePrincipalCredentials = DatabricksServicePrincipalCredentials

class DatabricksWorkspaceDescriptor(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OAUTH_SERVICE_URL_FIELD_NUMBER: builtins.int
    oauth_service_url: builtins.str
    def __init__(
        self,
        *,
        oauth_service_url: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["oauth_service_url", b"oauth_service_url"]) -> None: ...

global___DatabricksWorkspaceDescriptor = DatabricksWorkspaceDescriptor

class DatabricksAccessToken(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SECRET_FIELD_NUMBER: builtins.int
    @property
    def secret(self) -> qwak.administration.runtime_configuration.v0.creds.secret_pb2.SecretKeyRef: ...
    def __init__(
        self,
        *,
        secret: qwak.administration.runtime_configuration.v0.creds.secret_pb2.SecretKeyRef | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["location", b"location", "secret", b"secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["location", b"location", "secret", b"secret"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["location", b"location"]) -> typing_extensions.Literal["secret"] | None: ...

global___DatabricksAccessToken = DatabricksAccessToken



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import qwak.administration.runtime_configuration.v0.external.prometheus_config_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class VictoriaMetricsConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROMXY_FIELD_NUMBER: builtins.int
    @property
    def promxy(self) -> qwak.administration.runtime_configuration.v0.external.prometheus_config_pb2.PromxyConfiguration: ...
    def __init__(
        self,
        *,
        promxy: qwak.administration.runtime_configuration.v0.external.prometheus_config_pb2.PromxyConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["promxy", b"promxy", "proxy_type", b"proxy_type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["promxy", b"promxy", "proxy_type", b"proxy_type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["proxy_type", b"proxy_type"]) -> typing_extensions.Literal["promxy"] | None: ...

global___VictoriaMetricsConfiguration = VictoriaMetricsConfiguration



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import qwak.models.models_pb2
import qwak.projects.jfrog_project_spec_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ProjectRequest(google.protobuf.message.Message):
    """Request message for project operations"""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CREATE_PROJECT_REQUEST_FIELD_NUMBER: builtins.int
    DELETE_PROJECT_REQUEST_FIELD_NUMBER: builtins.int
    UPDATE_PROJECT_REQUEST_FIELD_NUMBER: builtins.int
    GET_PROJECT_REQUEST_FIELD_NUMBER: builtins.int
    LIST_PROJECTS_REQUEST_FIELD_NUMBER: builtins.int
    @property
    def create_project_request(self) -> global___CreateProjectRequest: ...
    @property
    def delete_project_request(self) -> global___DeleteProjectRequest: ...
    @property
    def update_project_request(self) -> global___UpdateProjectRequest: ...
    @property
    def get_project_request(self) -> global___GetProjectRequest: ...
    @property
    def list_projects_request(self) -> global___ListProjectsRequest: ...
    def __init__(
        self,
        *,
        create_project_request: global___CreateProjectRequest | None = ...,
        delete_project_request: global___DeleteProjectRequest | None = ...,
        update_project_request: global___UpdateProjectRequest | None = ...,
        get_project_request: global___GetProjectRequest | None = ...,
        list_projects_request: global___ListProjectsRequest | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["create_project_request", b"create_project_request", "delete_project_request", b"delete_project_request", "get_project_request", b"get_project_request", "list_projects_request", b"list_projects_request", "request", b"request", "update_project_request", b"update_project_request"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["create_project_request", b"create_project_request", "delete_project_request", b"delete_project_request", "get_project_request", b"get_project_request", "list_projects_request", b"list_projects_request", "request", b"request", "update_project_request", b"update_project_request"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["request", b"request"]) -> typing_extensions.Literal["create_project_request", "delete_project_request", "update_project_request", "get_project_request", "list_projects_request"] | None: ...

global___ProjectRequest = ProjectRequest

class CreateProjectRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_NAME_FIELD_NUMBER: builtins.int
    PROJECT_DESCRIPTION_FIELD_NUMBER: builtins.int
    JFROG_SPEC_FIELD_NUMBER: builtins.int
    project_name: builtins.str
    """Project name to create"""
    project_description: builtins.str
    """Project description"""
    @property
    def jfrog_spec(self) -> qwak.projects.jfrog_project_spec_pb2.ModelRepositoryJFrogSpec:
        """jfrog spec"""
    def __init__(
        self,
        *,
        project_name: builtins.str = ...,
        project_description: builtins.str = ...,
        jfrog_spec: qwak.projects.jfrog_project_spec_pb2.ModelRepositoryJFrogSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["jfrog_spec", b"jfrog_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["jfrog_spec", b"jfrog_spec", "project_description", b"project_description", "project_name", b"project_name"]) -> None: ...

global___CreateProjectRequest = CreateProjectRequest

class CreateProjectResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    @property
    def project(self) -> global___ProjectSpec:
        """Info about the creation process"""
    def __init__(
        self,
        *,
        project: global___ProjectSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["project", b"project"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["project", b"project"]) -> None: ...

global___CreateProjectResponse = CreateProjectResponse

class ListProjectsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ListProjectsRequest = ListProjectsRequest

class ListProjectsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECTS_FIELD_NUMBER: builtins.int
    @property
    def projects(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ProjectSpec]:
        """List of projects"""
    def __init__(
        self,
        *,
        projects: collections.abc.Iterable[global___ProjectSpec] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["projects", b"projects"]) -> None: ...

global___ListProjectsResponse = ListProjectsResponse

class GetProjectRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_ID_FIELD_NUMBER: builtins.int
    PROJECT_NAME_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    """Id of the project to get"""
    project_name: builtins.str
    """Name of the project to get"""
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
        project_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_id", b"project_id", "project_name", b"project_name"]) -> None: ...

global___GetProjectRequest = GetProjectRequest

class GetProjectResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_FIELD_NUMBER: builtins.int
    @property
    def project(self) -> global___Project:
        """The project object"""
    def __init__(
        self,
        *,
        project: global___Project | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["project", b"project"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["project", b"project"]) -> None: ...

global___GetProjectResponse = GetProjectResponse

class DeleteProjectRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_ID_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    """Id of the project to delete"""
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_id", b"project_id"]) -> None: ...

global___DeleteProjectRequest = DeleteProjectRequest

class DeleteProjectResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INFO_FIELD_NUMBER: builtins.int
    info: builtins.str
    """Info about the deleting process"""
    def __init__(
        self,
        *,
        info: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["info", b"info"]) -> None: ...

global___DeleteProjectResponse = DeleteProjectResponse

class UpdateProjectRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_ID_FIELD_NUMBER: builtins.int
    PROJECT_SPEC_CONFIG_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    """The project id to update"""
    @property
    def project_spec_config(self) -> global___ProjectSpecConfiguration:
        """The project specification to update"""
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
        project_spec_config: global___ProjectSpecConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["project_spec_config", b"project_spec_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_id", b"project_id", "project_spec_config", b"project_spec_config"]) -> None: ...

global___UpdateProjectRequest = UpdateProjectRequest

class UpdateProjectResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_SPEC_FIELD_NUMBER: builtins.int
    @property
    def project_spec(self) -> global___ProjectSpec:
        """The updated project specification"""
    def __init__(
        self,
        *,
        project_spec: global___ProjectSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["project_spec", b"project_spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_spec", b"project_spec"]) -> None: ...

global___UpdateProjectResponse = UpdateProjectResponse

class Project(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SPEC_FIELD_NUMBER: builtins.int
    MODELS_FIELD_NUMBER: builtins.int
    MODELS_BRIEFS_FIELD_NUMBER: builtins.int
    @property
    def spec(self) -> global___ProjectSpec:
        """The project specification"""
    @property
    def models(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.models.models_pb2.Model]:
        """List of models owned by the project"""
    @property
    def models_briefs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.models.models_pb2.ModelBrief]:
        """List of models owned by the project"""
    def __init__(
        self,
        *,
        spec: global___ProjectSpec | None = ...,
        models: collections.abc.Iterable[qwak.models.models_pb2.Model] | None = ...,
        models_briefs: collections.abc.Iterable[qwak.models.models_pb2.ModelBrief] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["spec", b"spec"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["models", b"models", "models_briefs", b"models_briefs", "spec", b"spec"]) -> None: ...

global___Project = Project

class ProjectSpec(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _Status:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _StatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ProjectSpec._Status.ValueType], builtins.type):  # noqa: F821
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        INVALID: ProjectSpec._Status.ValueType  # 0
        DISABLED: ProjectSpec._Status.ValueType  # 1
        SUSPENDED: ProjectSpec._Status.ValueType  # 2
        ACTIVE: ProjectSpec._Status.ValueType  # 3

    class Status(_Status, metaclass=_StatusEnumTypeWrapper):
        """Project Status
        Active - Live active project
        Suspended - Suspended project
        Disabled - Deleted Project
        """

    INVALID: ProjectSpec.Status.ValueType  # 0
    DISABLED: ProjectSpec.Status.ValueType  # 1
    SUSPENDED: ProjectSpec.Status.ValueType  # 2
    ACTIVE: ProjectSpec.Status.ValueType  # 3

    PROJECT_ID_FIELD_NUMBER: builtins.int
    PROJECT_NAME_FIELD_NUMBER: builtins.int
    PROJECT_DESCRIPTION_FIELD_NUMBER: builtins.int
    PROJECT_STATUS_FIELD_NUMBER: builtins.int
    CREATED_AT_FIELD_NUMBER: builtins.int
    CREATED_BY_FIELD_NUMBER: builtins.int
    LAST_MODIFIED_AT_FIELD_NUMBER: builtins.int
    LAST_MODIFIED_BY_FIELD_NUMBER: builtins.int
    MODELS_COUNT_FIELD_NUMBER: builtins.int
    MODELS_ACTIVE_FIELD_NUMBER: builtins.int
    JFROG_SETTINGS_FIELD_NUMBER: builtins.int
    project_id: builtins.str
    """The project id"""
    project_name: builtins.str
    """The project name"""
    project_description: builtins.str
    """Description of the project"""
    project_status: global___ProjectSpec.Status.ValueType
    """Project Status"""
    @property
    def created_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Timestamp of creation"""
    created_by: builtins.str
    """UserId that created the project"""
    @property
    def last_modified_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Last modified timestamp the project"""
    last_modified_by: builtins.str
    """Last modified userId"""
    models_count: builtins.int
    """Count of active(status not disabled) models in the project"""
    models_active: builtins.int
    """Count of deployed models in the project"""
    @property
    def jfrog_settings(self) -> global___ModelRepositoryJFrogSettings:
        """Jfrog model repository settings"""
    def __init__(
        self,
        *,
        project_id: builtins.str = ...,
        project_name: builtins.str = ...,
        project_description: builtins.str = ...,
        project_status: global___ProjectSpec.Status.ValueType = ...,
        created_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        created_by: builtins.str = ...,
        last_modified_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
        last_modified_by: builtins.str = ...,
        models_count: builtins.int = ...,
        models_active: builtins.int = ...,
        jfrog_settings: global___ModelRepositoryJFrogSettings | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["created_at", b"created_at", "jfrog_settings", b"jfrog_settings", "last_modified_at", b"last_modified_at"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["created_at", b"created_at", "created_by", b"created_by", "jfrog_settings", b"jfrog_settings", "last_modified_at", b"last_modified_at", "last_modified_by", b"last_modified_by", "models_active", b"models_active", "models_count", b"models_count", "project_description", b"project_description", "project_id", b"project_id", "project_name", b"project_name", "project_status", b"project_status"]) -> None: ...

global___ProjectSpec = ProjectSpec

class ProjectSpecConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PROJECT_NAME_FIELD_NUMBER: builtins.int
    PROJECT_DESCRIPTION_FIELD_NUMBER: builtins.int
    project_name: builtins.str
    """The project name"""
    project_description: builtins.str
    """Description of the project"""
    def __init__(
        self,
        *,
        project_name: builtins.str = ...,
        project_description: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["project_description", b"project_description", "project_name", b"project_name"]) -> None: ...

global___ProjectSpecConfiguration = ProjectSpecConfiguration

class ModelRepositoryJFrogSettings(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PULL_IMAGE_SECRET_FIELD_NUMBER: builtins.int
    JFROG_PROJECT_DETAILS_FIELD_NUMBER: builtins.int
    JFROG_REPOSITORIES_DETAILS_FIELD_NUMBER: builtins.int
    @property
    def pull_image_secret(self) -> global___ModelRepositoryPullImageSecret:
        """jfrog image pull secret info"""
    @property
    def jfrog_project_details(self) -> global___ModelRepositoryJFrogProjectDetails:
        """jfrog project details"""
    @property
    def jfrog_repositories_details(self) -> global___ModelRepositoryArtifactoryDetails:
        """jfrog repositories details"""
    def __init__(
        self,
        *,
        pull_image_secret: global___ModelRepositoryPullImageSecret | None = ...,
        jfrog_project_details: global___ModelRepositoryJFrogProjectDetails | None = ...,
        jfrog_repositories_details: global___ModelRepositoryArtifactoryDetails | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["jfrog_project_details", b"jfrog_project_details", "jfrog_repositories_details", b"jfrog_repositories_details", "pull_image_secret", b"pull_image_secret"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["jfrog_project_details", b"jfrog_project_details", "jfrog_repositories_details", b"jfrog_repositories_details", "pull_image_secret", b"pull_image_secret"]) -> None: ...

global___ModelRepositoryJFrogSettings = ModelRepositoryJFrogSettings

class ModelRepositoryPullImageSecret(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PULL_IMAGE_SECRET_NAME_FIELD_NUMBER: builtins.int
    pull_image_secret_name: builtins.str
    """The pull image secret name"""
    def __init__(
        self,
        *,
        pull_image_secret_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["pull_image_secret_name", b"pull_image_secret_name"]) -> None: ...

global___ModelRepositoryPullImageSecret = ModelRepositoryPullImageSecret

class ModelRepositoryJFrogProjectDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JFROG_PROJECT_KEY_FIELD_NUMBER: builtins.int
    jfrog_project_key: builtins.str
    """The jfrog project key"""
    def __init__(
        self,
        *,
        jfrog_project_key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["jfrog_project_key", b"jfrog_project_key"]) -> None: ...

global___ModelRepositoryJFrogProjectDetails = ModelRepositoryJFrogProjectDetails

class ModelRepositoryArtifactoryDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ARTIFACT_REPOSITORY_DETAILS_FIELD_NUMBER: builtins.int
    DATASET_REPOSITORY_DETAILS_FIELD_NUMBER: builtins.int
    DOCKER_REPOSITORY_DETAILS_FIELD_NUMBER: builtins.int
    @property
    def artifact_repository_details(self) -> global___ArtifactoryRepositoryDetails:
        """The jfrog artifact repository details"""
    @property
    def dataset_repository_details(self) -> global___ArtifactoryRepositoryDetails:
        """The jfrog dataset repository details"""
    @property
    def docker_repository_details(self) -> global___ArtifactoryRepositoryDetails:
        """The jfrog docker repository details"""
    def __init__(
        self,
        *,
        artifact_repository_details: global___ArtifactoryRepositoryDetails | None = ...,
        dataset_repository_details: global___ArtifactoryRepositoryDetails | None = ...,
        docker_repository_details: global___ArtifactoryRepositoryDetails | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["artifact_repository_details", b"artifact_repository_details", "dataset_repository_details", b"dataset_repository_details", "docker_repository_details", b"docker_repository_details"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["artifact_repository_details", b"artifact_repository_details", "dataset_repository_details", b"dataset_repository_details", "docker_repository_details", b"docker_repository_details"]) -> None: ...

global___ModelRepositoryArtifactoryDetails = ModelRepositoryArtifactoryDetails

class ArtifactoryRepositoryDetails(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    REPOSITORY_KEY_FIELD_NUMBER: builtins.int
    repository_key: builtins.str
    """The repository key"""
    def __init__(
        self,
        *,
        repository_key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["repository_key", b"repository_key"]) -> None: ...

global___ArtifactoryRepositoryDetails = ArtifactoryRepositoryDetails



import json
import os
from typing import Optional

from requests import Response
from tqdm.auto import tqdm
from tqdm.utils import CallbackIOWrapper
from urllib3 import Retry

from frogml_storage.logging import logger
from frogml_storage.utils import join_url
from frogml_storage.constants import CHECKSUM_SHA2_HEADER
from frogml_storage.models.entity_manifest import Checksums, EntityManifest
from frogml_storage.models.frogml_entity_type_info import FrogMLEntityTypeInfo
from frogml_storage.http import HTTPClient
from frogml_storage.models import DownloadContext


class StartTransactionResponse:
    files_upload_path: str
    lead_upload_path: str
    dependencies_upload_path: str
    code_upload_path: str
    transaction_id: str

    def __init__(
        self,
        files_upload_path,
        lead_upload_path,
        dependencies_upload_path,
        code_upload_path,
        transaction_id,
    ):
        self.files_upload_path = files_upload_path
        self.lead_upload_path = lead_upload_path
        self.dependencies_upload_path = dependencies_upload_path
        self.code_upload_path = code_upload_path
        self.transaction_id = transaction_id


class ArtifactoryApi:
    def __init__(self, uri, auth=None, http_client=None):
        self.uri = uri
        if http_client is not None:
            self.http_client = http_client
        else:
            self.auth = auth
            self.http_client = HTTPClient(auth=auth)

    def start_transaction(
        self,
        entity_type_info: FrogMLEntityTypeInfo,
        repository: str,
        entity_name: str,
        version: Optional[str],
    ) -> StartTransactionResponse:
        """
        Initializes an upload. Returns transaction ID and upload path
        """
        if version is None:
            start_transaction_url = (
                f"{self.uri}/api/machinelearning/{repository}/"
                f"{entity_type_info.entity_type}/{entity_name}/start-transaction"
            )
        else:
            start_transaction_url = (
                f"{self.uri}/api/machinelearning/{repository}/{entity_type_info.entity_type}"
                f"/{entity_name}/start-transaction/{version}"
            )
        try:
            response = self.http_client.post(start_transaction_url)
            response.raise_for_status()
            files_upload_path = response.json()["filesUploadPath"]
            lead_upload_path = response.json()["leadUploadPath"]
            dependencies_upload_path = response.json()["dependenciesUploadPath"]
            code_upload_path = response.json()["codeUploadPath"]
            transaction_id = response.json()["transactionId"]
        except Exception as exception:
            err = (
                f"Error occurred while trying to start an upload transaction for "
                f"{entity_type_info.entity_type}: '{entity_name}'"
                f" Error: '{exception}'"
            )
            logger.error(err, exc_info=False)
            raise exception
        return StartTransactionResponse(
            files_upload_path=files_upload_path,
            lead_upload_path=lead_upload_path,
            dependencies_upload_path=dependencies_upload_path,
            code_upload_path=code_upload_path,
            transaction_id=transaction_id,
        )

    def end_transaction(
        self,
        entity_type: FrogMLEntityTypeInfo,
        repository: str,
        entity_name: str,
        entity_manifest: EntityManifest,
        transaction_id: str,
        version: str,
        properties: Optional[dict[str, str]],
    ) -> None:
        """
        Upload model-manifest.json | dataset-manifest.json file, makes the model | dataset available in the repository
        """
        filename = entity_type.metadata_file_name

        url = join_url(
            self.uri,
            "api",
            "machinelearning",
            repository,
            entity_type.entity_type,
            "entity-manifest",
            entity_name,
            version,
            transaction_id,
        )

        json_entity_manifest = entity_manifest.to_json()
        self.upload_entity_manifest(
            entity_type=entity_type,
            filename=filename,
            payload=json_entity_manifest,
            url=url,
            properties=properties,
        )

    def download_file(self, args: DownloadContext) -> None:
        filename = os.path.basename(args.target_path)
        try:
            url = f"{self.uri}/{args.repo_key}/{args.source_url}"
            with self.http_client.get(url=url, stream=True) as response:
                response.raise_for_status()
                total_size = int(response.headers.get("content-length", 0))
                with open(args.target_path, "wb") as file:
                    with self.__initialize_progress_bar(total_size, filename) as pbar:
                        for chunk in response.iter_content(chunk_size=8192):
                            if chunk:
                                file.write(chunk)
                                pbar.update(len(chunk))

        except Exception as exception:
            self.__handle_download_exception(exception, args.target_path, filename)

    def get_entity_manifest(
        self,
        entity_type_info: FrogMLEntityTypeInfo,
        repository: str,
        entity_name: str,
        namespace: Optional[str],
        version: Optional[str],
    ) -> dict:
        url = join_url(
            self.uri,
            "api",
            "machinelearning",
            repository,
            entity_type_info.entity_type,
            "entity-manifest",
            namespace,
            entity_name,
            version,
        )
        try:
            with self.http_client.get(url=url) as r:
                r.raise_for_status()
                return r.json()
        except Exception as exception:
            err = f"Error occurred while trying to get {entity_type_info.entity_type} info file. Error: '{exception}'"
            logger.error(err, exc_info=False)
            raise exception

    @staticmethod
    def __handle_download_exception(
        exception: Exception, target_path: str, filename: str
    ) -> None:
        if os.path.exists(target_path):
            os.remove(target_path)
        err = f"Error occurred while trying to download file: '{filename}' Error: '{exception}'"
        logger.error(err, exc_info=False)
        raise exception

    def get_artifact_checksum(self, download_context: DownloadContext) -> str:
        url = f"{self.uri}/{download_context.repo_key}/{download_context.source_url}"
        try:
            with self.http_client.head(url=url, stream=True) as response:
                response.raise_for_status()
                return response.headers.get(CHECKSUM_SHA2_HEADER)

        except Exception as exception:
            logger.error(exception.__cause__, exc_info=False)
            raise exception

    def upload_entity_manifest(
        self,
        entity_type: FrogMLEntityTypeInfo,
        filename: str,
        payload: str,
        url: str,
        properties: Optional[dict[str, str]],
        stream: bool = False,
    ) -> None:
        body_part_name = f"{entity_type.body_part_stream}"

        try:
            files = {
                f"{body_part_name}": (
                    f"{body_part_name}",
                    payload,
                    "application/octet-stream",
                ),  # Include the InputStream
                "additionalData": (
                    "additionalData",
                    json.dumps(properties),
                    "application/octet-stream",
                ),  # Include the object
            }
            with self.http_client.put(url=url, files=files, stream=stream) as response:
                response.raise_for_status()
        except Exception as exception:
            err = f"Error occurred while trying to upload file: '{filename}' Error: '{exception}'"
            logger.error(err, exc_info=False)
            raise exception

    def upload_file(self, file_path: str, url: str) -> None:
        wrapped_file = None
        try:
            file_size = os.stat(file_path).st_size
            with (
                self.__initialize_progress_bar(file_size, file_path) as pbar,
                open(file_path, "rb") as file,
            ):
                wrapped_file = CallbackIOWrapper(pbar.update, file, "read")
                with self.http_client.put(url=url, payload=wrapped_file) as response:
                    response.raise_for_status()
        except Exception as exception:
            err = f"Error occurred while trying to upload file: '{file_path}' Error: '{exception}'"
            logger.error(err, exc_info=False)
            raise type(exception)(f"{err} File: {file_path}") from exception
        finally:
            if wrapped_file is not None:
                wrapped_file.close()

    def checksum_deployment(
        self,
        checksum: Checksums,
        url: str,
        full_path: str,
        stream: bool = False,
    ) -> bool:
        response = self.http_client.put(
            url=url,
            headers={"X-Checksum-Sha256": checksum.sha2, "X-Checksum-Deploy": "true"},
            stream=stream,
        )
        if response.status_code != 200 and response.status_code != 201:
            return False
        else:
            file_size = os.path.getsize(full_path)
            pbar = self.__initialize_progress_bar(file_size, full_path)
            pbar.update(file_size)
            pbar.close()
            return True

    @staticmethod
    def __initialize_progress_bar(total_size: int, filename: str) -> tqdm:
        return tqdm(
            total=total_size, unit="B", unit_scale=True, desc=filename, initial=0
        )

    def encrypt_password(self) -> Response:
        """
        returns encrypted password as text
        """
        return self.http_client.get(
            url=join_url(self.uri, "/api/security/encryptedPassword")
        )

    def ping(self) -> Response:
        """
        Sends a ping to Artifactory to validate login status
        """
        url = join_url(self.uri, "api/system/ping")
        return self.http_client.get(url=url)

    def get_artifactory_version(self) -> Response:
        return self.http_client.get(url=join_url(self.uri, "/api/system/version"))

    def create_machinelearning_local_repo(self, repo_name: str) -> Response:
        data = {
            "rclass": "local",
            "packageType": "machinelearning",
        }
        return self.http_client.put(
            url=join_url(self.uri, "/api/repositories/" + repo_name), json=data
        )

    def delete_frogml_local_repo(self, repo_name: str) -> Response:
        return self.http_client.delete(
            url=join_url(self.uri, "/api/repositories/" + repo_name)
        )


class RetryWithLog(Retry):
    """
    Adding extra logs before making a retry request
    """

    def __init__(self, *args, **kwargs):
        history = kwargs.get("history")
        if history is not None:
            logger.debug(f"Error: ${history[-1].error}\nretrying...")
        super().__init__(*args, **kwargs)



from datetime import datetime, timezone
from typing import Optional

from google.protobuf.timestamp_pb2 import Timestamp as ProtoTimestamp


def datetime_to_pts(dtime: Optional[datetime]) -> Optional[ProtoTimestamp]:
    """
    converts a python datetime to Protobuf Timestamp
    @param dtime: python datetime.datetime
    @return: if the input is None, returns None. else converts to proto timestamp in utc timezone
    """
    if not dtime:
        return None
    res: ProtoTimestamp = ProtoTimestamp()
    res.FromDatetime(dtime.replace(tzinfo=timezone.utc))

    return res



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/user_application/v0/user_application.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n/qwak/user_application/v0/user_application.proto\x12\x18qwak.user_application.v0\x1a\x1cgoogle/protobuf/struct.proto\".\n\x04Spec\x12&\n\x05value\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\"0\n\x06Status\x12&\n\x05value\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct*\xba\x03\n\x04Type\x12!\n\x1dINVALID_USER_APPLICATION_TYPE\x10\x00\x12\x10\n\x0cREMOTE_BUILD\x10\x01\x12\x1f\n\x17\x42\x41TCH_FEATURE_PROCESSOR\x10\x02\x1a\x02\x08\x01\x12#\n\x1bSTREAMING_FEATURE_PROCESSOR\x10\x03\x1a\x02\x08\x01\x12\x18\n\x14\x42\x41TCH_INFERENCE_TASK\x10\x04\x12\x14\n\x10MODEL_DEPLOYMENT\x10\x05\x12\x11\n\rMODEL_MONITOR\x10\x06\x12\x1b\n\x17\x44\x45PLOYMENT_MICROSERVICE\x10\x07\x12\x0f\n\x0b\x41PPLICATION\x10\x08\x12\x14\n\x10\x42\x41TCH_FEATURESET\x10\t\x12\x18\n\x14\x43USTOMER_ALERT_STACK\x10\n\x12\x11\n\rMODEL_TRAFFIC\x10\x0b\x12\x18\n\x14STREAMING_FEATURESET\x10\x0c\x12$\n STREAMING_AGGREGATION_FEATURESET\x10\r\x12\x1a\n\x16MLOPS_MODEL_DEPLOYMENT\x10\x0e\x12\x17\n\x13MLOPS_MODEL_TRAFFIC\x10\x0f\x12\x0e\n\nAI_GATEWAY\x10\x10\x42\xaa\x01\n\x1f\x63om.qwak.ai.user_application.v0P\x01Z\x84\x01github.com/qwak-ai/qwak-platform/services/core/go/admiral/user-application-value-api/pb/qwak/user_application/v0;user_application_v0b\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.user_application.v0.user_application_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.qwak.ai.user_application.v0P\001Z\204\001github.com/qwak-ai/qwak-platform/services/core/go/admiral/user-application-value-api/pb/qwak/user_application/v0;user_application_v0'
  _TYPE.values_by_name["BATCH_FEATURE_PROCESSOR"]._options = None
  _TYPE.values_by_name["BATCH_FEATURE_PROCESSOR"]._serialized_options = b'\010\001'
  _TYPE.values_by_name["STREAMING_FEATURE_PROCESSOR"]._options = None
  _TYPE.values_by_name["STREAMING_FEATURE_PROCESSOR"]._serialized_options = b'\010\001'
  _globals['_TYPE']._serialized_start=206
  _globals['_TYPE']._serialized_end=648
  _globals['_SPEC']._serialized_start=107
  _globals['_SPEC']._serialized_end=153
  _globals['_STATUS']._serialized_start=155
  _globals['_STATUS']._serialized_end=203
# @@protoc_insertion_point(module_scope)



from typing import Callable, Tuple

import grpc
from qwak.inner.const import QwakConstants

_SIGNATURE_HEADER_KEY = "authorization"


class Auth0Client(grpc.AuthMetadataPlugin):
    def __init__(self):
        self._auth_client = None

    def __call__(
        self,
        context: grpc.AuthMetadataContext,
        callback: Callable[[Tuple[Tuple[str, str]], None], None],
    ):
        """Implements authentication by passing metadata to a callback.

        Args:
            context: An AuthMetadataContext providing information on the RPC that
                the plugin is being called to authenticate.
            callback: A callback that accepts a tuple of metadata key/value pairs and a None
                parameter.
        """
        # Get token from Auth0 client
        token = self.get_token()
        metadata = ((_SIGNATURE_HEADER_KEY, f"Bearer {token}"),)
        callback(metadata, None)

    def get_token(self) -> str:
        """Get the authentication token."""
        if not self._auth_client:
            from qwak.inner.tool.auth import Auth0ClientBase

            self._auth_client = Auth0ClientBase()
        return self._auth_client.get_token()


class FrogMLGrpcClient(grpc.AuthMetadataPlugin):
    def __init__(self):
        self._auth_client = None

    def __call__(
        self,
        context: grpc.AuthMetadataContext,
        callback: Callable[[Tuple[Tuple[str, str]], None], None],
    ):
        """Implements authentication by passing metadata to a callback.

        Args:
            context: An AuthMetadataContext providing information on the RPC that
                the plugin is being called to authenticate.
            callback: A callback that accepts a tuple of metadata key/value pairs and a None
                parameter.
        """
        token = self.get_token()
        jfrog_tenant_id = self._auth_client.get_tenant_id()
        metadata = (
            (_SIGNATURE_HEADER_KEY, f"Bearer {token}"),
            (QwakConstants.JFROG_TENANT_HEADER_KEY.lower(), jfrog_tenant_id),
        )
        callback(metadata, None)

    def get_token(self) -> str:
        """Get the authentication token."""
        if not self._auth_client:
            from qwak.inner.tool.auth import FrogMLAuthClient

            self._auth_client = FrogMLAuthClient()
        return self._auth_client.get_token()



import contextlib
import os
import shutil
import tempfile
import zipfile
from io import BytesIO
from pathlib import Path
from shutil import make_archive
from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Union
from uuid import uuid4

import requests
from qwak.clients.feature_store.management_client import FeatureRegistryClient
from qwak.exceptions import QwakException
from qwak.feature_store._common.value import (
    UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG,
)
from requests import HTTPError, Response
from urllib.parse import urlparse, ParseResult

ZIP_FUNCTION_CONTENT_TYPE = "text/plain"

if TYPE_CHECKING:
    from qwak.feature_store.data_sources.streaming.kafka.deserialization import (
        Deserializer,
    )
    from qwak.feature_store.feature_sets.transformations.transformations import (
        BaseTransformation,
    )


def zip_source_code_dir(base_path: Path) -> Path:
    ignore_patterns = ["*.pyc", "__pycache__"]
    return zip_tree_to_temp_dir(base_path=base_path, ignore_patterns=ignore_patterns)


def zip_tree_to_temp_dir(base_path: Path, ignore_patterns=None) -> Path:
    if ignore_patterns is None:
        ignore_patterns = []

    full_path_to_archive = None
    parent_base_path = base_path.parent if base_path.is_file() else base_path
    tempdir = tempfile.mkdtemp()
    dst: Path = Path(os.path.join(tempdir, parent_base_path.name))
    try:
        shutil.copytree(
            src=parent_base_path,
            dst=dst,
            ignore=shutil.ignore_patterns(*ignore_patterns),
        )

        full_path_to_archive: Path = zip_path_dir_recursively(
            p=dst, dest=tempfile.mkdtemp()
        )
        return full_path_to_archive
    except Exception as e:
        raise QwakException("Failed to copy and zip directory") from e
    finally:
        shutil.rmtree(tempdir, ignore_errors=True)
        if full_path_to_archive:
            shutil.rmtree(full_path_to_archive, ignore_errors=True)


def zip_path_dir_recursively(p: Path, dest: Path) -> Path:
    root_dir: Path = p.parent if p.is_file() else p
    full_path_to_archive: str = make_archive(
        base_name=os.path.join(dest, str(uuid4())),
        format="zip",
        root_dir=root_dir.parent,
        base_dir=root_dir.name,
    )
    return Path(full_path_to_archive)


def zip_function_with_base_folder(
    functions: List[Callable], fs_object_module_dir: Path = None
) -> bytes:
    """
    Prepare zip bytes of the UDF functions and it's parent folder
    In case the functions param is a Callable it will write the pkled code to "function.pkl" (original flow)
    If a list of functions it will write each function to {function_name}.pkl
    Args:
        functions: udf functions
        fs_object_module_dir: parent directory path of the registered feature store object
    Returns:
        bytes representing the zip bytes of the udf functions pkl and module
    """
    try:
        import cloudpickle
    except Exception:
        raise QwakException(
            f"Missing required 'cloudpickle' dependency. {UPDATE_QWAK_SDK_WITH_FEATURE_STORE_EXTRA_MSG}"
        )

    in_memory = BytesIO()
    with chdir(fs_object_module_dir):
        zf = zipfile.ZipFile(in_memory, mode="w")
        with zf:
            if fs_object_module_dir:
                for path in Path.cwd().glob("**/*"):
                    str_path = (
                        f"./{path.relative_to(Path.cwd())}"
                        if path.is_file()
                        else str(path.resolve())
                    )
                    zf.write(str_path, "code" + "/" + str_path)
            for function in functions:
                zf.writestr(f"{function.__name__}.pkl", cloudpickle.dumps(function))
        in_memory.seek(0)
    return in_memory.read()


@contextlib.contextmanager
def chdir(dirname: Path = None):
    """
    chdir to given path and return to previous cwd when done
    Args:
        dirname: dir to chdir into
    """
    current_dir = Path.cwd()
    try:
        if dirname is not None:
            os.chdir(dirname)
        yield
    finally:
        os.chdir(current_dir)


def get_featureset_presign_url(
    features_manager_client: FeatureRegistryClient,
    feature_store_object_name: str,
    feature_store_object_name_suffix: str,
    transformation,
) -> str:
    return features_manager_client.get_presigned_url_v1(
        feature_set_name=feature_store_object_name + feature_store_object_name_suffix,
        transformation=transformation._to_proto(),
    )


def get_data_source_presign_url_v1(
    features_manager_client: FeatureRegistryClient,
    data_source_object_name: str,
    data_source_name_suffix: str,
) -> str:
    return features_manager_client.get_data_source_presigned_url_v1(
        data_source_name=data_source_object_name, object_name=data_source_name_suffix
    )


def upload_artifact(
    feature_store_object_name: str,
    feature_store_object_name_suffix: str,
    functions: List[Callable],
    feature_module_dir: Path,
    features_manager_client: FeatureRegistryClient,
    artifact_object: Union["BaseTransformation", "Deserializer"],
) -> str:
    """
    This function is used for uploading the function code in case a UDF type transformation was used (Pandas/Koalas/Pyspark)
    If a transformation was supplied, meaning we're running for at least batch, will use the SaaS adapted
    function to get the presign URL.
    """
    uploaded_artifact_url: str

    # this is done here to avoid cyclic imports
    from qwak.feature_store.data_sources.streaming.kafka.deserialization import (
        Deserializer,
    )
    from qwak.feature_store.feature_sets.transformations.transformations import (
        BaseTransformation,
    )

    if isinstance(artifact_object, BaseTransformation):
        presign_url = get_featureset_presign_url(
            features_manager_client=features_manager_client,
            feature_store_object_name=feature_store_object_name,
            feature_store_object_name_suffix=feature_store_object_name_suffix,
            transformation=artifact_object,
        )
    elif isinstance(artifact_object, Deserializer):
        presign_url = get_data_source_presign_url_v1(
            features_manager_client=features_manager_client,
            data_source_object_name=feature_store_object_name,
            data_source_name_suffix=feature_store_object_name_suffix,
        )
    else:
        raise QwakException(
            f"Got an unsupported artifact type: {type(artifact_object)}"
        )

    uploaded_artifact_url = base_upload_function_code(
        presign_url, functions, feature_module_dir
    )

    return uploaded_artifact_url


def put_files_content(
    url: str,
    content: bytes,
    content_type: str,
    extra_headers: Optional[Dict[str, str]] = None,
) -> None:
    """
    uploads files content to s3 using presigned url or to jfrog artifactory using url and authentication headers
    """
    extra_headers = extra_headers or {}
    try:
        http_response: Response = requests.put(
            url,
            headers={"content-type": content_type, **extra_headers},
            data=content,
            timeout=300,
        )
        http_response.raise_for_status()
    except HTTPError as e:
        raise QwakException(
            f"Error while uploading artifact. status code: {e.response.status_code} with reason: {e.response.reason}"
        ) from e


def upload_to_s3(presign_url: str, in_memory_zip: bytes, content_type: str) -> str:
    parsed_url: ParseResult = urlparse(presign_url)
    extra_headers: Dict[str, str] = (
        {"x-ms-blob-type": "BlockBlob"}
        if parsed_url.hostname.endswith(".blob.core.windows.net")
        else None
    )
    put_files_content(
        url=presign_url,
        content=in_memory_zip,
        content_type=content_type,
        extra_headers=extra_headers,
    )

    postfix: str = parsed_url.path
    bucket: str = parsed_url.hostname.split(".s3.")[0]
    s3_path: str = f"s3://{bucket}{postfix}"  # noqa: E231
    return s3_path


def base_upload_function_code(
    presign_url: str,
    functions: List[Callable],
    feature_module_dir: Path,
) -> str:
    """
    get presign url and prepare a zip file with the functions code and it's module than uploads it to clients' s3 bucket
    Args:
        presign_url: presign url
        functions: udf functions
        feature_module_dir: parent directory path of the feature
    Returns:
        Full path to the zip file uploaded
    """
    in_memory_zip = zip_function_with_base_folder(functions, feature_module_dir)
    return upload_to_s3(
        presign_url=presign_url,
        in_memory_zip=in_memory_zip,
        content_type=ZIP_FUNCTION_CONTENT_TYPE,
    )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/inference/feedback/feedback.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&qwak/inference/feedback/feedback.proto\x12\x18\x63om.qwak.ai.feedback.api\x1a\x1fgoogle/protobuf/timestamp.proto\"\xbc\x01\n\x13\x41\x63tualValuesRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x30\n\x06\x65ntity\x18\x02 \x01(\x0b\x32 .com.qwak.ai.feedback.api.Entity\x12\x32\n\x07\x61\x63tuals\x18\x03 \x01(\x0b\x32!.com.qwak.ai.feedback.api.Actuals\x12-\n\ttimestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xde\x01\n\x18\x45nrichedActualValuesData\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x30\n\x06\x65ntity\x18\x02 \x01(\x0b\x32 .com.qwak.ai.feedback.api.Entity\x12\x32\n\x07\x61\x63tuals\x18\x03 \x01(\x0b\x32!.com.qwak.ai.feedback.api.Actuals\x12\x1b\n\x13qwak_environment_id\x18\x04 \x01(\t\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"%\n\x06\x45ntity\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"%\n\x07\x41\x63tuals\x12\x0b\n\x03tag\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x03(\t\"\x98\x01\n\x14\x41\x63tualValuesResponse\x12J\n\x06status\x18\x01 \x01(\x0e\x32:.com.qwak.ai.feedback.api.ActualValuesResponse.ValueStatus\"4\n\x0bValueStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07\x46\x41ILURE\x10\x01\x12\x0b\n\x07SUCCESS\x10\x02\"\xf1\x02\n\x18\x43onfigureFeedbackRequest\x12\x10\n\x08model_id\x18\x01 \x01(\t\x12\x1f\n\x17\x61nalytics_entity_column\x18\x02 \x01(\t\x12\x1e\n\x16\x66\x65\x65\x64\x62\x61\x63k_entity_column\x18\x03 \x01(\t\x12\x41\n\x0f\x66\x65\x65\x64\x62\x61\x63k_config\x18\x04 \x03(\x0b\x32(.com.qwak.ai.feedback.api.FeedbackConfig\x12P\n\nmodel_type\x18\x05 \x01(\x0e\x32<.com.qwak.ai.feedback.api.ConfigureFeedbackRequest.ModelType\"m\n\tModelType\x12\x16\n\x12INVALID_MODEL_TYPE\x10\x00\x12\x19\n\x15\x42INARY_CLASSIFICATION\x10\x01\x12\x1d\n\x19MULTICLASS_CLASSIFICATION\x10\x02\x12\x0e\n\nREGRESSION\x10\x03\";\n\x0e\x46\x65\x65\x64\x62\x61\x63kConfig\x12\x15\n\routput_column\x18\x01 \x01(\t\x12\x12\n\nactual_tag\x18\x02 \x01(\t\"\xa2\x01\n\x19\x43onfigureFeedbackResponse\x12O\n\x06status\x18\x01 \x01(\x0e\x32?.com.qwak.ai.feedback.api.ConfigureFeedbackResponse.ValueStatus\"4\n\x0bValueStatus\x12\x0b\n\x07INVALID\x10\x00\x12\x0b\n\x07\x46\x41ILURE\x10\x01\x12\x0b\n\x07SUCCESS\x10\x02\x32\xfe\x01\n\x0f\x46\x65\x65\x64\x62\x61\x63kService\x12m\n\x0cPostFeedback\x12-.com.qwak.ai.feedback.api.ActualValuesRequest\x1a..com.qwak.ai.feedback.api.ActualValuesResponse\x12|\n\x11\x43onfigureFeedback\x12\x32.com.qwak.ai.feedback.api.ConfigureFeedbackRequest\x1a\x33.com.qwak.ai.feedback.api.ConfigureFeedbackResponseB\x1a\n\x18\x63om.qwak.ai.feedback.apib\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.inference.feedback.feedback_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\030com.qwak.ai.feedback.api'
  _globals['_ACTUALVALUESREQUEST']._serialized_start=102
  _globals['_ACTUALVALUESREQUEST']._serialized_end=290
  _globals['_ENRICHEDACTUALVALUESDATA']._serialized_start=293
  _globals['_ENRICHEDACTUALVALUESDATA']._serialized_end=515
  _globals['_ENTITY']._serialized_start=517
  _globals['_ENTITY']._serialized_end=554
  _globals['_ACTUALS']._serialized_start=556
  _globals['_ACTUALS']._serialized_end=593
  _globals['_ACTUALVALUESRESPONSE']._serialized_start=596
  _globals['_ACTUALVALUESRESPONSE']._serialized_end=748
  _globals['_ACTUALVALUESRESPONSE_VALUESTATUS']._serialized_start=696
  _globals['_ACTUALVALUESRESPONSE_VALUESTATUS']._serialized_end=748
  _globals['_CONFIGUREFEEDBACKREQUEST']._serialized_start=751
  _globals['_CONFIGUREFEEDBACKREQUEST']._serialized_end=1120
  _globals['_CONFIGUREFEEDBACKREQUEST_MODELTYPE']._serialized_start=1011
  _globals['_CONFIGUREFEEDBACKREQUEST_MODELTYPE']._serialized_end=1120
  _globals['_FEEDBACKCONFIG']._serialized_start=1122
  _globals['_FEEDBACKCONFIG']._serialized_end=1181
  _globals['_CONFIGUREFEEDBACKRESPONSE']._serialized_start=1184
  _globals['_CONFIGUREFEEDBACKRESPONSE']._serialized_end=1346
  _globals['_CONFIGUREFEEDBACKRESPONSE_VALUESTATUS']._serialized_start=696
  _globals['_CONFIGUREFEEDBACKRESPONSE_VALUESTATUS']._serialized_end=748
  _globals['_FEEDBACKSERVICE']._serialized_start=1349
  _globals['_FEEDBACKSERVICE']._serialized_end=1603
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/model_group/model_group.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from _qwak_proto.qwak.model_group import model_group_repository_details_pb2 as qwak_dot_model__group_dot_model__group__repository__details__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"qwak/model_group/model_group.proto\x12\x1cqwak.model_groups.management\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x35qwak/model_group/model_group_repository_details.proto\"<\n\"MigrateAccountSystemSecretsRequest\x12\x16\n\x0emodel_group_id\x18\x01 \x01(\t\"7\n#MigrateAccountSystemSecretsResponse\x12\x10\n\x08migrated\x18\x01 \x01(\x08\"\xa3\x01\n)GetModelGroupSystemPullImageSecretRequest\x12v\n*model_group_system_pull_secret_identifiers\x18\x01 \x03(\x0b\x32\x42.qwak.model_groups.management.ModelGroupSystemPullSecretIdentifier\"p\n$ModelGroupSystemPullSecretIdentifier\x12\x1a\n\x10model_group_name\x18\x01 \x01(\tH\x00\x12\x12\n\naccount_id\x18\x02 \x01(\tB\x18\n\x16model_group_identifier\"m\n!SystemPullSecretInfoNamesResponse\x12H\n\x0csecret_infos\x18\x01 \x03(\x0b\x32\x32.qwak.model_groups.management.SystemPullSecretInfo\"_\n\x14SystemPullSecretInfo\x12\x18\n\x10model_group_name\x18\x01 \x01(\t\x12\x19\n\x11secret_pull_image\x18\x02 \x01(\t\x12\x12\n\naccount_id\x18\x03 \x01(\t\"Y\n\"CreateIfNotExistsModelGroupRequest\x12\x18\n\x10model_group_name\x18\x01 \x01(\t\x12\x19\n\x11jfrog_project_key\x18\x02 \x01(\t\"\xdf\x01\n\x1bModelGroupBriefInfoResponse\x12\x16\n\x0emodel_group_id\x18\x01 \x01(\t\x12\x1f\n\x17model_group_description\x18\x02 \x01(\t\x12:\n\x16model_group_created_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12K\n\x12repository_details\x18\x04 \x03(\x0b\x32/.qwak.model_groups.management.RepositoryDetails\"\x95\x01\n&CreateRepositoriesForModelGroupRequest\x12\x18\n\x10model_group_uuid\x18\x01 \x01(\t\x12Q\n\x12repository_details\x18\x02 \x03(\x0b\x32\x35.qwak.model_groups.management.RemoteRepositoryDetails\"\x82\x01\n\'CreateRepositoriesForModelGroupResponse\x12W\n\x1e\x61rtifactory_repository_details\x18\x01 \x03(\x0b\x32/.qwak.model_groups.management.RepositoryDetails\"O\n\x18GetModelGroupInfoRequest\x12\x18\n\x10model_group_name\x18\x01 \x01(\t\x12\x19\n\x11jfrog_project_key\x18\x02 \x01(\t2\xca\x06\n\x1bModelGroupManagementService\x12\x9a\x01\n\x1b\x43reateIfNotExistsModelGroup\x12@.qwak.model_groups.management.CreateIfNotExistsModelGroupRequest\x1a\x39.qwak.model_groups.management.ModelGroupBriefInfoResponse\x12\xae\x01\n\x1f\x43reateRepositoriesForModelGroup\x12\x44.qwak.model_groups.management.CreateRepositoriesForModelGroupRequest\x1a\x45.qwak.model_groups.management.CreateRepositoriesForModelGroupResponse\x12\x86\x01\n\x11GetModelGroupInfo\x12\x36.qwak.model_groups.management.GetModelGroupInfoRequest\x1a\x39.qwak.model_groups.management.ModelGroupBriefInfoResponse\x12\xae\x01\n\"GetModelGroupSystemPullImageSecret\x12G.qwak.model_groups.management.GetModelGroupSystemPullImageSecretRequest\x1a?.qwak.model_groups.management.SystemPullSecretInfoNamesResponse\x12\xa2\x01\n\x1bMigrateAccountSystemSecrets\x12@.qwak.model_groups.management.MigrateAccountSystemSecretsRequest\x1a\x41.qwak.model_groups.management.MigrateAccountSystemSecretsResponseBL\n&com.qwak.ai.management.model.group.apiB ModelGroupManagementServiceProtoP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.model_group.model_group_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n&com.qwak.ai.management.model.group.apiB ModelGroupManagementServiceProtoP\001'
  _globals['_MIGRATEACCOUNTSYSTEMSECRETSREQUEST']._serialized_start=156
  _globals['_MIGRATEACCOUNTSYSTEMSECRETSREQUEST']._serialized_end=216
  _globals['_MIGRATEACCOUNTSYSTEMSECRETSRESPONSE']._serialized_start=218
  _globals['_MIGRATEACCOUNTSYSTEMSECRETSRESPONSE']._serialized_end=273
  _globals['_GETMODELGROUPSYSTEMPULLIMAGESECRETREQUEST']._serialized_start=276
  _globals['_GETMODELGROUPSYSTEMPULLIMAGESECRETREQUEST']._serialized_end=439
  _globals['_MODELGROUPSYSTEMPULLSECRETIDENTIFIER']._serialized_start=441
  _globals['_MODELGROUPSYSTEMPULLSECRETIDENTIFIER']._serialized_end=553
  _globals['_SYSTEMPULLSECRETINFONAMESRESPONSE']._serialized_start=555
  _globals['_SYSTEMPULLSECRETINFONAMESRESPONSE']._serialized_end=664
  _globals['_SYSTEMPULLSECRETINFO']._serialized_start=666
  _globals['_SYSTEMPULLSECRETINFO']._serialized_end=761
  _globals['_CREATEIFNOTEXISTSMODELGROUPREQUEST']._serialized_start=763
  _globals['_CREATEIFNOTEXISTSMODELGROUPREQUEST']._serialized_end=852
  _globals['_MODELGROUPBRIEFINFORESPONSE']._serialized_start=855
  _globals['_MODELGROUPBRIEFINFORESPONSE']._serialized_end=1078
  _globals['_CREATEREPOSITORIESFORMODELGROUPREQUEST']._serialized_start=1081
  _globals['_CREATEREPOSITORIESFORMODELGROUPREQUEST']._serialized_end=1230
  _globals['_CREATEREPOSITORIESFORMODELGROUPRESPONSE']._serialized_start=1233
  _globals['_CREATEREPOSITORIESFORMODELGROUPRESPONSE']._serialized_end=1363
  _globals['_GETMODELGROUPINFOREQUEST']._serialized_start=1365
  _globals['_GETMODELGROUPINFOREQUEST']._serialized_end=1444
  _globals['_MODELGROUPMANAGEMENTSERVICE']._serialized_start=1447
  _globals['_MODELGROUPMANAGEMENTSERVICE']._serialized_end=2289
# @@protoc_insertion_point(module_scope)



from __future__ import annotations

from abc import ABC, abstractmethod


class BuildLogger(ABC):
    verbose: int
    json_logs: bool

    @abstractmethod
    def error(self, line: str) -> None:
        pass

    @abstractmethod
    def exception(self, line: str, e: BaseException) -> None:
        pass

    @abstractmethod
    def warning(self, line: str) -> None:
        pass

    @abstractmethod
    def info(self, line: str) -> None:
        pass

    @abstractmethod
    def debug(self, line: str) -> None:
        pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/logging/log_filter.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dqwak/logging/log_filter.proto\x12\x14qwak.logging.service\x1a\x1fgoogle/protobuf/timestamp.proto\"\xe3\x01\n\x0cSearchFilter\x12\x38\n\x0flog_text_filter\x18\x01 \x01(\x0b\x32\x1d.qwak.logging.service.LogTextH\x00\x12\x46\n\x13\x61nd_operator_filter\x18\x02 \x01(\x0b\x32\'.qwak.logging.service.AndOperatorFilterH\x00\x12G\n\x18log_emitting_time_filter\x18\x03 \x01(\x0b\x32#.qwak.logging.service.LogTimeFilterH\x00\x42\x08\n\x06\x66ilter\"\x1b\n\x07LogText\x12\x10\n\x08\x63ontains\x18\x01 \x01(\t\"p\n\rLogTimeFilter\x12.\n\nafter_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12/\n\x0b\x62\x65\x66ore_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"O\n\x11\x41ndOperatorFilter\x12:\n\x0esearch_filters\x18\x01 \x03(\x0b\x32\".qwak.logging.service.SearchFilterB,\n\x1e\x63om.qwak.ai.logging.reader.apiP\x01Z\x08/loggingb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.logging.log_filter_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\036com.qwak.ai.logging.reader.apiP\001Z\010/logging'
  _globals['_SEARCHFILTER']._serialized_start=89
  _globals['_SEARCHFILTER']._serialized_end=316
  _globals['_LOGTEXT']._serialized_start=318
  _globals['_LOGTEXT']._serialized_end=345
  _globals['_LOGTIMEFILTER']._serialized_start=347
  _globals['_LOGTIMEFILTER']._serialized_end=459
  _globals['_ANDOPERATORFILTER']._serialized_start=461
  _globals['_ANDOPERATORFILTER']._serialized_end=540
# @@protoc_insertion_point(module_scope)



from typing import Optional

import grpc

from _qwak_proto.qwak.administration.cluster.v2.cluster_pb2 import (
    ClusterState,
)
from _qwak_proto.qwak.administration.cluster.v2.cluster_service_pb2 import (
    CreateClusterCreationRequestResponse,
    DeleteClusterResponse,
    GetClusterResponse,
    GetClusterStateResponse,
    ListClustersResponse,
    UpdateClusterConfigurationResponse,
)
from _qwak_proto.qwak.administration.cluster.v2.cluster_service_pb2_grpc import (
    ClusterServiceServicer,
)


class ClusterV2ServiceMock(ClusterServiceServicer):
    def __init__(self):
        super().__init__()
        self.__create_cluster_response: Optional[
            CreateClusterCreationRequestResponse
        ] = None
        self.__create_cluster_error: Optional[grpc.StatusCode] = None

        self.__get_cluster_state_response: Optional[GetClusterStateResponse] = None
        self.__get_cluster_state_error: Optional[grpc.StatusCode] = None

        self.__get_cluster_response: Optional[GetClusterResponse] = None
        self.__get_cluster_error: Optional[grpc.StatusCode] = None

        self.__list_clusters_response: Optional[ListClustersResponse] = None
        self.__list_clusters_error: Optional[grpc.StatusCode] = None

        self.__update_cluster_response: Optional[UpdateClusterConfigurationResponse] = (
            None
        )
        self.__update_cluster_error: Optional[grpc.StatusCode] = None

        self.__delete_cluster_response: Optional[DeleteClusterResponse] = None
        self.__delete_cluster_error: Optional[grpc.StatusCode] = None

    def given_create_cluster(
        self,
        response: Optional[CreateClusterCreationRequestResponse] = None,
        error_code: Optional[grpc.StatusCode] = None,
    ):
        self.__create_cluster_response = response
        self.__create_cluster_error = error_code

    def given_get_cluster_state(
        self,
        response: Optional[GetClusterStateResponse] = None,
        error_code: Optional[grpc.StatusCode] = None,
    ):
        self.__get_cluster_state_response = response
        self.__get_cluster_state_error = error_code

    def given_get_cluster(
        self,
        response: Optional[GetClusterResponse] = None,
        error_code: Optional[grpc.StatusCode] = None,
    ):
        self.__get_cluster_response = response
        self.__get_cluster_error = error_code

    def given_list_clusters(
        self,
        response: Optional[ListClustersResponse] = None,
        error_code: Optional[grpc.StatusCode] = None,
    ):
        self.__list_clusters_response = response
        self.__list_clusters_error = error_code

    def given_update_cluster(
        self,
        response: Optional[UpdateClusterConfigurationResponse] = None,
        error_code: Optional[grpc.StatusCode] = None,
    ):
        self.__update_cluster_response = response
        self.__update_cluster_error = error_code

    def given_delete_cluster(
        self,
        response: Optional[DeleteClusterResponse] = None,
        error_code: Optional[grpc.StatusCode] = None,
    ):
        self.__delete_cluster_response = response
        self.__delete_cluster_error = error_code

    def CreateClusterCreationRequest(self, request, context):
        if self.__create_cluster_error:
            context.set_code(self.__create_cluster_error)
            context.set_details("Failed to create cluster")
            return CreateClusterCreationRequestResponse()
        if self.__create_cluster_response:
            return self.__create_cluster_response
        return CreateClusterCreationRequestResponse(request_id="mock-request-id")

    def GetClusterState(self, request, context):
        if self.__get_cluster_state_error:
            context.set_code(self.__get_cluster_state_error)
            context.set_details("Failed to get cluster state")
            return GetClusterStateResponse()
        if self.__get_cluster_state_response:
            return self.__get_cluster_state_response
        return GetClusterStateResponse(cluster_state=ClusterState())

    def GetCluster(self, request, context):
        if self.__get_cluster_error:
            context.set_code(self.__get_cluster_error)
            context.set_details("Failed to get cluster")
            return GetClusterResponse()
        if self.__get_cluster_response:
            return self.__get_cluster_response
        return GetClusterResponse(cluster_state=ClusterState())

    def ListClusters(self, request, context):
        if self.__list_clusters_error:
            context.set_code(self.__list_clusters_error)
            context.set_details("Failed to list clusters")
            return ListClustersResponse()
        if self.__list_clusters_response:
            return self.__list_clusters_response
        return ListClustersResponse()

    def UpdateClusterConfiguration(self, request, context):
        if self.__update_cluster_error:
            context.set_code(self.__update_cluster_error)
            context.set_details("Failed to update cluster")
            return UpdateClusterConfigurationResponse()
        if self.__update_cluster_response:
            return self.__update_cluster_response
        return UpdateClusterConfigurationResponse(request_id="mock-request-id")

    def DeleteCluster(self, request, context):
        if self.__delete_cluster_error:
            context.set_code(self.__delete_cluster_error)
            context.set_details("Failed to delete cluster")
            return DeleteClusterResponse()
        if self.__delete_cluster_response:
            return self.__delete_cluster_response
        return DeleteClusterResponse(request_id="mock-request-id")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: qwak/fitness_service/fitness.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"qwak/fitness_service/fitness.proto\x12\x14qwak.fitness.service*e\n\x0ePurchaseOption\x12\x1b\n\x17INVALID_PURCHASE_OPTION\x10\x00\x12\x18\n\x14SPOT_PURCHASE_OPTION\x10\x01\x12\x1c\n\x18ONDEMAND_PURCHASE_OPTION\x10\x02\x42#\n\x1f\x63om.qwak.ai.fitness.service.apiP\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'qwak.fitness_service.fitness_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\037com.qwak.ai.fitness.service.apiP\001'
  _globals['_PURCHASEOPTION']._serialized_start=60
  _globals['_PURCHASEOPTION']._serialized_end=161
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from _qwak_proto.qwak.admiral.secret.v0 import system_secret_service_pb2 as qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2


class SystemSecretServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SetSystemSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SystemSecretService/SetSystemSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetSystemSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetSystemSecretResponse.FromString,
                )
        self.DeleteSystemSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SystemSecretService/DeleteSystemSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteSystemSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteSystemSecretResponse.FromString,
                )
        self.GetSystemSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SystemSecretService/GetSystemSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetSystemSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetSystemSecretResponse.FromString,
                )
        self.SetAccountSystemSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SystemSecretService/SetAccountSystemSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetAccountSystemSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetAccountSystemSecretResponse.FromString,
                )
        self.GetAccountSystemSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SystemSecretService/GetAccountSystemSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetAccountSystemSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetAccountSystemSecretResponse.FromString,
                )
        self.DeleteAccountSystemSecret = channel.unary_unary(
                '/qwak.admiral.secret.v0.SystemSecretService/DeleteAccountSystemSecret',
                request_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteAccountSystemSecretRequest.SerializeToString,
                response_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteAccountSystemSecretResponse.FromString,
                )


class SystemSecretServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SetSystemSecret(self, request, context):
        """Set a value for a secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteSystemSecret(self, request, context):
        """Delete secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSystemSecret(self, request, context):
        """Get secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetAccountSystemSecret(self, request, context):
        """Create or Update a value for an account system secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetAccountSystemSecret(self, request, context):
        """Get account system secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteAccountSystemSecret(self, request, context):
        """Delete account level secret
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SystemSecretServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SetSystemSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.SetSystemSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetSystemSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetSystemSecretResponse.SerializeToString,
            ),
            'DeleteSystemSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteSystemSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteSystemSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteSystemSecretResponse.SerializeToString,
            ),
            'GetSystemSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSystemSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetSystemSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetSystemSecretResponse.SerializeToString,
            ),
            'SetAccountSystemSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.SetAccountSystemSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetAccountSystemSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetAccountSystemSecretResponse.SerializeToString,
            ),
            'GetAccountSystemSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.GetAccountSystemSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetAccountSystemSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetAccountSystemSecretResponse.SerializeToString,
            ),
            'DeleteAccountSystemSecret': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteAccountSystemSecret,
                    request_deserializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteAccountSystemSecretRequest.FromString,
                    response_serializer=qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteAccountSystemSecretResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'qwak.admiral.secret.v0.SystemSecretService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SystemSecretService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SetSystemSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SystemSecretService/SetSystemSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetSystemSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetSystemSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteSystemSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SystemSecretService/DeleteSystemSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteSystemSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteSystemSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSystemSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SystemSecretService/GetSystemSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetSystemSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetSystemSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetAccountSystemSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SystemSecretService/SetAccountSystemSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetAccountSystemSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.SetAccountSystemSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetAccountSystemSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SystemSecretService/GetAccountSystemSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetAccountSystemSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.GetAccountSystemSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteAccountSystemSecret(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/qwak.admiral.secret.v0.SystemSecretService/DeleteAccountSystemSecret',
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteAccountSystemSecretRequest.SerializeToString,
            qwak_dot_admiral_dot_secret_dot_v0_dot_system__secret__service__pb2.DeleteAccountSystemSecretResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class AccountAcceptedTerms(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TERMS_FIELD_NUMBER: builtins.int
    @property
    def terms(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Term]:
        """Account accepted terms"""
    def __init__(
        self,
        *,
        terms: collections.abc.Iterable[global___Term] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["terms", b"terms"]) -> None: ...

global___AccountAcceptedTerms = AccountAcceptedTerms

class Term(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PLATFORM_GENERAL_TERM_FIELD_NUMBER: builtins.int
    MODEL_LIBRARY_GENERAL_TERM_FIELD_NUMBER: builtins.int
    MODEL_LIBRARY_MODEL_TERM_FIELD_NUMBER: builtins.int
    @property
    def platform_general_term(self) -> global___PlatformGeneralTerm:
        """Platform general term"""
    @property
    def model_library_general_term(self) -> global___ModelLibraryGeneralTerm:
        """Model library general term"""
    @property
    def model_library_model_term(self) -> global___ModelLibraryModelTerm:
        """Model library model term"""
    def __init__(
        self,
        *,
        platform_general_term: global___PlatformGeneralTerm | None = ...,
        model_library_general_term: global___ModelLibraryGeneralTerm | None = ...,
        model_library_model_term: global___ModelLibraryModelTerm | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["model_library_general_term", b"model_library_general_term", "model_library_model_term", b"model_library_model_term", "platform_general_term", b"platform_general_term", "term", b"term"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["model_library_general_term", b"model_library_general_term", "model_library_model_term", b"model_library_model_term", "platform_general_term", b"platform_general_term", "term", b"term"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["term", b"term"]) -> typing_extensions.Literal["platform_general_term", "model_library_general_term", "model_library_model_term"] | None: ...

global___Term = Term

class PlatformGeneralTerm(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___PlatformGeneralTerm = PlatformGeneralTerm

class ModelLibraryGeneralTerm(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ModelLibraryGeneralTerm = ModelLibraryGeneralTerm

class ModelLibraryModelTerm(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    MODEL_REGISTRY_ID_FIELD_NUMBER: builtins.int
    model_registry_id: builtins.str
    """The id of the model registry"""
    def __init__(
        self,
        *,
        model_registry_id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["model_registry_id", b"model_registry_id"]) -> None: ...

global___ModelLibraryModelTerm = ModelLibraryModelTerm



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import qwak.ecosystem.v0.auth_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class AzureTemporaryCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    AZURE_DATABRICKS_SERVICE_TEMPORARY_CREDENTIALS_FIELD_NUMBER: builtins.int
    DATABRICKS_CREDENTIALS_FIELD_NUMBER: builtins.int
    AZURE_STORAGE_CREDENTIALS_FIELD_NUMBER: builtins.int
    @property
    def azure_databricks_service_temporary_credentials(self) -> global___AzureDatabricksServiceTemporaryCredentials: ...
    @property
    def databricks_credentials(self) -> global___DatabricksCredentials: ...
    @property
    def azure_storage_credentials(self) -> global___AzureStorageCredentials: ...
    def __init__(
        self,
        *,
        azure_databricks_service_temporary_credentials: global___AzureDatabricksServiceTemporaryCredentials | None = ...,
        databricks_credentials: global___DatabricksCredentials | None = ...,
        azure_storage_credentials: global___AzureStorageCredentials | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["azure_databricks_service_temporary_credentials", b"azure_databricks_service_temporary_credentials", "azure_storage_credentials", b"azure_storage_credentials", "databricks_credentials", b"databricks_credentials", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["azure_databricks_service_temporary_credentials", b"azure_databricks_service_temporary_credentials", "azure_storage_credentials", b"azure_storage_credentials", "databricks_credentials", b"databricks_credentials", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["azure_databricks_service_temporary_credentials", "databricks_credentials", "azure_storage_credentials"] | None: ...

global___AzureTemporaryCredentials = AzureTemporaryCredentials

class AzureDatabricksServiceTemporaryCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DATABRICKS_CREDENTIALS_FIELD_NUMBER: builtins.int
    AZURE_STORAGE_CREDENTIALS_FIELD_NUMBER: builtins.int
    @property
    def databricks_credentials(self) -> global___DatabricksCredentials: ...
    @property
    def azure_storage_credentials(self) -> global___AzureStorageCredentials: ...
    def __init__(
        self,
        *,
        databricks_credentials: global___DatabricksCredentials | None = ...,
        azure_storage_credentials: global___AzureStorageCredentials | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["azure_storage_credentials", b"azure_storage_credentials", "databricks_credentials", b"databricks_credentials"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["azure_storage_credentials", b"azure_storage_credentials", "databricks_credentials", b"databricks_credentials"]) -> None: ...

global___AzureDatabricksServiceTemporaryCredentials = AzureDatabricksServiceTemporaryCredentials

class DatabricksCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OAUTH_TOKEN_CREDENTIAL_FIELD_NUMBER: builtins.int
    @property
    def oauth_token_credential(self) -> qwak.ecosystem.v0.auth_pb2.OAuthTokenCredential: ...
    def __init__(
        self,
        *,
        oauth_token_credential: qwak.ecosystem.v0.auth_pb2.OAuthTokenCredential | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["oauth_token_credential", b"oauth_token_credential", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["oauth_token_credential", b"oauth_token_credential", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["oauth_token_credential"] | None: ...

global___DatabricksCredentials = DatabricksCredentials

class AzureStorageCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SAS_TOKEN_CREDENTIAL_FIELD_NUMBER: builtins.int
    @property
    def sas_token_credential(self) -> global___SASTokenCredentials: ...
    def __init__(
        self,
        *,
        sas_token_credential: global___SASTokenCredentials | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["sas_token_credential", b"sas_token_credential", "type", b"type"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["sas_token_credential", b"sas_token_credential", "type", b"type"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["type", b"type"]) -> typing_extensions.Literal["sas_token_credential"] | None: ...

global___AzureStorageCredentials = AzureStorageCredentials

class SASTokenCredentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TOKEN_FIELD_NUMBER: builtins.int
    EXPIRED_AT_FIELD_NUMBER: builtins.int
    token: builtins.str
    """The token to use for authentication"""
    @property
    def expired_at(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Expiration timestamp of the credentials"""
    def __init__(
        self,
        *,
        token: builtins.str = ...,
        expired_at: google.protobuf.timestamp_pb2.Timestamp | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["expired_at", b"expired_at"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["expired_at", b"expired_at", "token", b"token"]) -> None: ...

global___SASTokenCredentials = SASTokenCredentials



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import qwak.instance_template.instance_template_pb2
import sys

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class ListInstanceTemplatesRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPTIONAL_INSTANCE_FILTER_FIELD_NUMBER: builtins.int
    @property
    def optional_instance_filter(self) -> qwak.instance_template.instance_template_pb2.InstanceFilter: ...
    def __init__(
        self,
        *,
        optional_instance_filter: qwak.instance_template.instance_template_pb2.InstanceFilter | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["optional_instance_filter", b"optional_instance_filter"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["optional_instance_filter", b"optional_instance_filter"]) -> None: ...

global___ListInstanceTemplatesRequest = ListInstanceTemplatesRequest

class ListInstanceTemplatesResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INSTANCE_TEMPLATE_LIST_FIELD_NUMBER: builtins.int
    @property
    def instance_template_list(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[qwak.instance_template.instance_template_pb2.InstanceTemplateSpec]: ...
    def __init__(
        self,
        *,
        instance_template_list: collections.abc.Iterable[qwak.instance_template.instance_template_pb2.InstanceTemplateSpec] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["instance_template_list", b"instance_template_list"]) -> None: ...

global___ListInstanceTemplatesResponse = ListInstanceTemplatesResponse

class GetInstanceTemplateRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ID_FIELD_NUMBER: builtins.int
    id: builtins.str
    def __init__(
        self,
        *,
        id: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id", b"id"]) -> None: ...

global___GetInstanceTemplateRequest = GetInstanceTemplateRequest

class GetInstanceTemplateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    INSTANCE_TEMPLATE_FIELD_NUMBER: builtins.int
    @property
    def instance_template(self) -> qwak.instance_template.instance_template_pb2.InstanceTemplateSpec: ...
    def __init__(
        self,
        *,
        instance_template: qwak.instance_template.instance_template_pb2.InstanceTemplateSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["instance_template", b"instance_template"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["instance_template", b"instance_template"]) -> None: ...

global___GetInstanceTemplateResponse = GetInstanceTemplateResponse



from abc import ABC, abstractmethod
from dataclasses import dataclass, field
from datetime import datetime

from _qwak_proto.qwak.feature_store.features.feature_set_types_pb2 import (
    Backfill as ProtoBackfill,
)
from google.protobuf.timestamp_pb2 import Timestamp


class FillupMethodABC(ABC):
    @abstractmethod
    def to_proto(self):
        pass


class AsScheduled(FillupMethodABC):
    @classmethod
    def to_proto(cls):
        return ProtoBackfill.AS_SCHEDULED


class Snapshot(FillupMethodABC):
    @classmethod
    def to_proto(cls):
        return ProtoBackfill.SNAPSHOT


class BackfillType:
    Snapshot = Snapshot
    AsScheduled = AsScheduled


@dataclass
class Backfill:
    """
    A data class that holds the details of a backfilling operation.

    Attributes:
    -----------
    start_date: datetime
        The date from which the backfilling operation should commence.

    fill_up_method: FillupMethodABC (default: BackfillType.AsScheduled)
        The method to be used for the backfilling operation. This determines
        the way data will be filled up during the backfill process.
    """

    start_date: datetime
    fill_up_method: FillupMethodABC = field(default_factory=BackfillType.AsScheduled)

    def to_proto(self):
        proto_timestamp = Timestamp()
        proto_timestamp.FromDatetime(self.start_date)
        return ProtoBackfill(
            start_date=proto_timestamp, fillup_method=self.fill_up_method.to_proto()
        )

    @classmethod
    def from_proto(cls, proto: "ProtoBackfill"):
        backfill_mapping = {
            ProtoBackfill.FillUpMethod.AS_SCHEDULED: BackfillType.AsScheduled(),
            ProtoBackfill.FillUpMethod.SNAPSHOT: BackfillType.Snapshot(),
        }

        return cls(
            start_date=datetime.utcfromtimestamp(
                proto.start_date.seconds + proto.start_date.nanos / 1e9,
            ),
            fill_up_method=backfill_mapping.get(proto.fillup_method),
        )
